/*!
    @class		SineGrain
    @abstract   a class to generate a sine wave
    @discussion (description)
*/
public class SineGrain {
	sinosc s => pan2 p => dac;
	
	0.01 => static float time_step;
	
	/*!
      @function		perform
      @abstract		this is called to produce a wave form.
      @discussion (description)
      @param      (name) (description)
      @result     (description)
	*/
	fun void perform(float freq, float amp, float attack, float decay, float pan) {
		freq => s.freq;
		0.0 => s.gain;
		pan => p.pan;
		
		// the attack portion
		attack::ms => dur t;
		now + t => time t1;
		amp / (attack / time_step) => float gain_step;
		<<< t, t1, gain_step >>>;
		while(now < t1) {
			time_step::second => now;
			s.gain() + gain_step => s.gain;
			<<< s.gain() >>>;
		}
		time_step::second => now;
		
		// the decay portion
		decay::ms => t;
		now + t => t1;
		amp / (decay / time_step) => gain_step;
		<<< t, t1, gain_step >>>;
		while(now < t1) {
			time_step::second => now;
			s.gain() - gain_step => s.gain;
			<<< s.gain() >>>;
		}
		0.0 => s.gain;
		
		<<< "done..." >>>;
	}
}

/*
5.0 / 4.0 => float amp;
15000.0 => float attack;
1000.0 => float decay;

SineGrain sw;
spork ~ sw.perform(880.0, amp, attack, decay, .5);

new SineGrain @=> sw;
spork ~ sw.perform(660.0, amp, attack, decay, .5);

new SineGrain @=> sw;
spork ~ sw.perform(440.0, amp, attack, decay, .5);

new SineGrain @=> sw;
spork ~ sw.perform(1210.0, amp, attack, decay, .5);

(attack + decay)::ms + 5::second => dur t;

now + t => time end;

end => now;
*/
